from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
import undetected_chromedriver as uc
import random

# Short function to find element, using webdriver wait
def find_element(driver, by, value, timeout=10):
    return WebDriverWait(driver, timeout).until(
        EC.presence_of_element_located((by, value))
    )

def find_elements(driver, by, value, timeout=10):
    return WebDriverWait(driver, timeout).until(
        EC.presence_of_all_elements_located((by, value))
    )

# Simple function to scroll to bottom of screen.
def scroll_to_bottom(driver):
    driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")

def set_element_innertext(driver, element, text):
    driver.execute_script("arguments[0].innerText = arguments[1];", element, text)

def new_driver():
    options = uc.ChromeOptions()
    options.add_argument("--disable-notifications")  # Disable notifications
    options.add_argument("--disable-popup-blocking")  # Disable popup blocking
    driver = uc.Chrome(options=options)
    return driver

def gen_random_string(length=8):
    random_string = ''.join(random.choice('0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz') for i in range(length))
    return random_string



from youtube_selenium_py.youtube.get_all_video_stats_from_channel import get_all_video_stats_from_channel
from youtube_selenium_py.youtube.get_my_channel_handle import get_my_channel_handle

def get_my_videos_stats(driver):
    try:
        result = get_my_channel_handle(driver)
        if result['status'] == "success":
            channel_handle = result['channel_handle']
            driver = result['driver']
        else:
            driver = result['driver']
            raise Exception("Error: channel ID not found.")

        video_stats = get_all_video_stats_from_channel(channel_handle)
        return {
            "status": "success",
            "message": "Video stats found.",
            "video_stats": video_stats,
            "driver": driver,
        }
    
    except Exception as e:
        print("Error: " + str(e))

        with open("error.txt", "a") as error_file:
            error_file.write(str(e) + "\n")

        return {
            "status": "error",
            "message": "Video stats not found.",
            "driver": driver,
            "error": str(e),
        }



from pytube import YouTube

def download_video(video_id: str, absolute_path :str, filename: str):
    try:
        url = f"https://www.youtube.com/watch?v={video_id}"
        yt = YouTube(url)
       
        # Filter streams to get those with video and audio, sorted by resolution
        streams = yt.streams.filter(progressive=True, file_extension='mp4').order_by('resolution').desc()
        
        # Find the highest resolution stream that is less than or equal to 1080p
        stream = None
        for s in streams:
            if s.resolution and int(s.resolution.split('p')[0]) <= 1080:
                stream = s
                break
        
        if stream:
            # Download the video with the specified filename
            stream.download(output_path=absolute_path, filename=filename)
            print("Download completed successfully.")
            return {
                "status": "success",
                "message": "Video downloaded successfully.",
            }
        else:
            print("No suitable streams found.")
            return {
                "status": "error",
                "message": "No suitable streams found.",
                "error:": "No suitable streams found.",
            }
    except Exception as e:
        print("An error occurred while downloading the video.")

        with open("error.txt", "w") as f:
            f.write(str(e))

        return {
            "status": "error",
            "message": "Error downloading video.",
            "error": str(e),
        }



import unittest
import os
from youtube_selenium_py.classes import Youtube
import time
from dotenv import load_dotenv

load_dotenv()

test_email = os.getenv("TEST_EMAIL")
test_password = os.getenv("TEST_PASSWORD")

class TestListAllChannels(unittest.TestCase):

    def test_list_all_channels_success(self):
        if not test_email or not test_password:
            self.skipTest("Test email and password not provided")

        youtube = Youtube(email=test_email, password=test_password)
        result = youtube.list_all_channels()

        if result:
            print(result)
            self.assertEqual(result["status"], "success")
        else:
            raise Exception("Channel listing failed, result is None")
        
        time.sleep(5)
        youtube.close()



import unittest
from youtube_selenium_py.classes import Youtube 
import os
from dotenv import load_dotenv

load_dotenv()

test_email = os.getenv("TEST_EMAIL")
test_password = os.getenv("TEST_PASSWORD")

class TestCreateCommunityPostSubChannel(unittest.TestCase):

    def test_text_post(self):

        youtube = Youtube(email=test_email, password=test_password)
        youtube.switch_to_sub_channel(
            "Adonis Jamal"
        )
        result = youtube.create_community_post(
            community_post_title="Hello everyone, how is it going?",
            schedule={
                "date": "Apr 5, 2024",
                "time": "6:45 PM", # Only 15 minute increments (hour:0, hour:15, hour: 30, hour: 45)
                "GMT_timezone": "GMT-7" # Timezone of the schedule (GMT only)
            },
        )
        self.assertEqual(result['status'], 'success')
        youtube.close()



import requests
import math
import threading

def all_video_stats_from_channel(channel_id):

    print("Fetching all videos from channel...")
    get_channel_info = f"https://yt.lemnoslife.com/noKey/channels?id={channel_id}&part=contentDetails" 
    response = requests.get(get_channel_info)
    uploads_id = response.json()['items'][0]['contentDetails']['relatedPlaylists']['uploads']

    get_videos_url = f"https://yt.lemnoslife.com/noKey/playlistItems?playlistId={uploads_id}&part=snippet&maxResults=800"
    response = requests.get(get_videos_url)
    response_json = response.json()
    try:
        next_page_token = response_json.get('nextPageToken')
        total_results = response_json['pageInfo']['totalResults']

        total_pages = math.ceil(total_results / 50)

        all_videos = response.json()['items']

        for page in range(1, total_pages):
            next_page_url = f"{get_videos_url}&pageToken={next_page_token}"
            response = requests.get(next_page_url)
            next_page_token = response.json().get('nextPageToken')
            all_videos += response.json()['items']

        print(f"Total of {len(all_videos)} videos fetched.")
        return all_videos

    except:
        if response_json.get("error")['message'] == "The playlist identified with the request's <code>playlistId</code> parameter cannot be found.":
            print("Problem navigating to next page. Are you sure the channel has videos uploaded?")
            return []
        else:
            print("An error occurred.")
            raise Exception(response_json)

def chunk_list(lst, chunk_size):
    """
    Split a list into chunks of specified size.
    """
    return [lst[i:i + chunk_size] for i in range(0, len(lst), chunk_size)]

def get_video_stats_for_chunk(chunk, url_with_ids_appended, all_videos_data):
    """
    Fetch video statistics for a chunk of video IDs.
    """
    response = requests.get(url_with_ids_appended)
    response_json = response.json()
    videos = response_json['items']
    all_videos_data += videos

def get_all_video_stats(all_videos):
    print("Extracting all video ids...")

    get_videos_info_url="https://yt.lemnoslife.com/noKey/videos?part=snippet,statistics"
    if len(all_videos) == 0:
        return []
    all_videos_ids = [video['snippet']['resourceId']['videoId'] for video in all_videos]
    chunked_video_ids = chunk_list(all_videos_ids, 25)

    print(f"Total of {len(all_videos_ids)} video ids extracted...")

    all_videos_data = []
    threads = []

    print("Starting to fetch video statistics...")
    for i, chunk in enumerate(chunked_video_ids):
        url_with_ids_appended = f"{get_videos_info_url}&id={','.join(chunk)}"
        thread = threading.Thread(target=get_video_stats_for_chunk, args=(chunk, url_with_ids_appended, all_videos_data))
        thread.start()
        threads.append(thread)

    for thread in threads:
        thread.join()

    print(f"Total of {len(all_videos_data)} videos fetched.")
    return all_videos_data 

def get_all_video_stats_from_channel(channel_handle: str):

    print(f"Converting handle '{channel_handle}' to channel id...")
    channel_handle = channel_handle.replace("@", "")
    request_url = f"https://yt.lemnoslife.com/channels?handle=@{channel_handle}"
    response = requests.get(request_url)
    channel_id = response.json()['items'][0]['id']

    try:
        all_videos = all_video_stats_from_channel(channel_id)
        all_videos_with_stats = get_all_video_stats(all_videos)
        return {
            "all_video_stats": all_videos_with_stats,
            "status": "success",
            "message": "All video stats fetched successfully."
        }
    except Exception as e:
        print("An error occurred.")
        with open("error.txt", "w") as f:
            f.write(str(e))

        return {
            "status": "error",
            "message": "An error occurred. Please check errorr.txt for more details.",
            "error": str(e),
        }



import unittest
from youtube_selenium_py.classes import YoutubeData

class TestGetVideoStats(unittest.TestCase):

    def test_get_video_stats(self):
        youtube_data = YoutubeData()
        result = youtube_data.get_video_stats("0P1FGtdg8o8")

        if result:
            print(result)
            self.assertEqual(result["status"], "success")
        else:
            raise Exception("Getting video stats failed, result is None")



from selenium.webdriver.common.by import By
from youtube_selenium_py.utils import find_element, find_elements 
import time

def list_all_channels(
    driver
):
    try:
        time.sleep(3)
        print("1. Navigating to youtube.com...")
        if driver.current_url != "https://www.youtube.com/":
            driver.get("https://www.youtube.com/")
        avatar_button = find_element(driver, By.CSS_SELECTOR, "button[id='avatar-btn']")
        avatar_button.click()

        switch_account_button = find_element(driver, By.XPATH, "/html/body/ytd-app/ytd-popup-container/tp-yt-iron-dropdown/div/ytd-multi-page-menu-renderer/div[3]/div[1]/yt-multi-page-menu-section-renderer[1]/div[2]/ytd-compact-link-renderer[2]")
        switch_account_button.click()

        channels_items = find_elements(driver, By.CSS_SELECTOR, "ytd-account-item-renderer[thumbnail-size='48']")

        channels_list = []
        print("2. Listing all channels...")
        for i, channel_item in enumerate(channels_items):
            channel_name, channel_handle = channel_item.find_elements(By.CSS_SELECTOR, "yt-formatted-string")[:2]
            print(f"[{i+1}] Channel Name:", channel_name.text)
            print(f"[{i+1}] Channel Handle:", channel_handle.text)
            channels_list.append({
                "channel_name": channel_name.text,
                "channel_handle": channel_handle.text
            })

        print("3. Channels listed successfully.")
        return {
            "channels_list": channels_list,
            "status": "success",
            "message": "Channels listed successfully",
            "driver": driver,
        }

    except Exception as e:
        print(e)
        with open("error.txt", "w") as f:
            f.write(str(e))
        return {
            "status": "error",
            "message": str(e),
            "driver": driver
        } 



from typing import List
import time
from selenium.webdriver.common.by import By
from youtube_selenium_py.utils import find_element

def create_sub_channels(
    driver,
    sub_channels_names: List[str],
):
    try:
        time.sleep(5)
        for i, sub_channel_name in enumerate(sub_channels_names):
            print(f"[{i + 1}] Creating sub channel: {sub_channel_name}")
            print("1. Navigating to account settings...")
            driver.get("https://youtube.com/account")

            # Get the anchor element with the text inside "Create a new channel"
            try:
                create_channel_link = find_element(driver, By.XPATH, "//a[contains(text(), 'Create a new channel')]")
                create_channel_link.click()
            except:
                add_or_manage_channels_link = find_element(driver, By.XPATH, "//a[contains(text(), 'Add or manage your channel(s)')]")
                add_or_manage_channels_link.click()

                create_channel_link = find_element(driver, By.CSS_SELECTOR, "a[aria-label='Create a channel']")
                create_channel_link.click()
            
            print("2. Entering channel name...")
            channel_name_input = find_element(driver, By.CSS_SELECTOR, "input[id='PlusPageName']")
            channel_name_input.click()
            channel_name_input.send_keys(sub_channel_name)
        
            print("3. Clicking on checkbox...")
            checkbox_input = find_element(driver, By.CSS_SELECTOR, "input[type='checkbox']")
            checkbox_input.click()

            print("4. Creating channel by pressing submit button...")
            submit_input_button = find_element(driver, By.CSS_SELECTOR, "input[id='submitbutton']")
            submit_input_button.click()

            print("5. Waiting for the channel to be created...")
            time.sleep(10)
            
            print(f"Sub channel with name '{sub_channel_name}' created successfully.")

        return {
            "status": "success",
            "message": "Sub channels created successfully.",
            "driver": driver
        }
    except Exception as e:
        print(f"Error: {e}")
        with open("error.txt", "w") as f:
            f.write(str(e))

        return {
            "status": "error",
            "message": "An error occurred while creating sub channels.",
            "driver": driver
        }




import unittest
from youtube_selenium_py.classes import Youtube
import os
from dotenv import load_dotenv

load_dotenv()

test_email = os.getenv("TEST_EMAIL")
test_password = os.getenv("TEST_PASSWORD")

test_video_path = os.path.abspath("./tests/assets/test_vid.mp4")

class TestCreateVideoSubChannel(unittest.TestCase):

    def test_create_video_success(self):
        # Calling the create_video function with valid parameters
        youtube = Youtube(email=test_email, password=test_password)
        youtube.switch_to_sub_channel(
            "Adonis Jamal"
        )
        result = youtube.create_video(
            absolute_video_path=test_video_path,
            video_title="Test Title",
            video_description="Test video description",
            video_thumbnail_absolute_path=os.path.abspath("./tests/assets/channel_banner.png"),
            video_schedule_date="Apr 5, 2024",
            video_schedule_time="6:45 PM", # 24-hour format, only 15 minute increments alowed (hour:15, hour:30: hour:45: hour:00)
        )

        # Asserting that the result is a success
        self.assertEqual(result["status"], "success")
        youtube.close()



import requests

def get_channel_id(channel_handle: str):
    try:
        print(f"Converting handle '{channel_handle}' to channel id...")
        channel_handle = channel_handle.replace("@", "")
        request_url = f"https://yt.lemnoslife.com/channels?handle=@{channel_handle}"
        response = requests.get(request_url)
        channel_id = response.json()['items'][0]['id']
        
        print(f"Successfully converted handle '{channel_handle}' to channel id: {channel_id}.")

        return {
            "status": "success",
            "channel_id": channel_id,
        }

    except Exception as e:
        print(f"Error while converting handle '{channel_handle}' to channel id.")
        with open("error.txt", "w") as f:
            f.write(str(e))

        return {
            "status": "error",
            "message": "An error occurred while converting handle to channel id.",
            "error": str(e),
        }



import unittest
from youtube_selenium_py.classes import YoutubeData
import os

class TestDownloadVideo(unittest.TestCase):

    def test_download_video_success(self):
        youtube_data = YoutubeData()
        path = "./tests/assets"
        absolute_path = os.path.abspath(path)
        result = youtube_data.download_video("vEf2z1YeXws", absolute_path, "test_download_video.mp4")

        if result:
            self.assertEqual(result["status"], "success")
        else:
            raise Exception("Video download failed, result is None")



from typing import Optional, List
import youtube_selenium_py.youtube as youtube
from youtube_selenium_py.utils import new_driver
from dataclasses import dataclass

@dataclass
class Youtube:
    email: Optional[str] = None
    password: Optional[str] = None
    absolute_chromium_profile_path: Optional[str] = None
    cookies: Optional[list] = None
    
    def __post_init__(self):
        if self.email is None and self.password is None and self.absolute_chromium_profile_path is None and self.cookies is None:
            raise Exception("Please provide either email and password or cookies and absolute_chromium_profile_path.")
        self.driver = new_driver()  # Assuming new_driver() is defined somewhere
        self.sign_in()

    def sign_in(self):
        driver = youtube.sign_into_youtube_channel(self.driver, self.email, self.password, self.cookies, self.absolute_chromium_profile_path)
        # Set class driver to driver
        self.driver = driver
    
    def create_channel(
        self, 
    ):
        """
        Function to create a Youtube channel. 
        Returns:
        - example success return object: {
            "status": "success",
            "channel_id": channel_id, 
            "channel_name": default_channel_name,
            "channel_handle": default_channel_handle,
            "message": "Channel created successfully", 
            "cookies": driver.get_cookies(),
            "driver": driver
        }
        - example error return object: {
            "status": "error",
            "message": "An error occurred while creating channel."
            "error": error_message,
            "driver": driver
        }
        """
        result = youtube.create_channel(
            self.driver,
        )
        self.driver = result['driver']

        print(result)

        return result

    def edit_channel(
        self,
        channel_name: str,
        channel_handle: str,
        channel_description: str,
        profile_picture_path: str,
        banner_picture_path: str, watermark_picture_path: str,
        contact_email_path: str,
        links: list,
    ):
        """
        Function to edit channel

        Parameters:
        - channel_name (str): name of the channel.
        - channel_handle (str): channel handle/username.
        - channel_description (str): description of the channel.
        - profile_picture_path (str): path of the profile picture of the channel.
        - banner_picture_path (str): path of the banner picture of the channel.
        - watermark_picture_path (str): path of the watermark picture of the channel.
        - contact_email_path (str): path of the contact email of the channel.
        - links (list[dict]): list of links to be added to the channel.
            - Format of links:
            [
                {
                    "title": str,
                    "url": str,
                },
                {
                    "title": str,
                    "url": str,
                },
                ...
            ]
        Returns:
        - example success return object: {
            "status": "success",
            "message": "Channel edited successfully",
            "channel_id": channel_id,
            "channel_name": channel_name,
            "channel_handle": channel_handle, # if channel_handle is unavailable, it will use a random string as the new channel handle.
            "driver": driver
        }
        - example error return object: {
            "status": "error",
            "message": "An error occurred while editing channel.",
            "error": str(e),
            "driver": driver,
        }
        """
        result = youtube.edit_channel(
            self.driver,
            channel_name,
            channel_handle,
            channel_description,
            profile_picture_path,
            banner_picture_path,
            watermark_picture_path,
            contact_email_path,
            links
        )
        self.driver = result['driver']
        return result


    def create_video(
        self,
        absolute_video_path: str,
        video_title: str,
        video_description: str,
        video_thumbnail_absolute_path: Optional[str] = None,
        video_schedule_date: Optional[str] = None,
        video_schedule_time: Optional[str] = None,
    ):
        """
        Function to upload a video to a Youtube channel. 

        Parameters:
        - absolute_video_path (str): absolute path of video to be uploaded.
        - video_title (str): title of the video.
        - video_description (str): description of the video.
        - video_thumbnail_absolute_path (Optional[str]): absolute path of the thumbnail of the video.
        - video_schedule_date (Optional[str] -> format: 'Apr 5, 2024'): date to schedule the video.
        - video_schedule_time (Optional[str] -> format: '6:45 PM'): time to schedule the video.
        
        Returns:
        - example success return object: {
            "status": "success",
            "channel_id": channel_id,
            "video_id": video_id,
            "message": "Video uploaded successfully",
            "driver": driver,
        }
        - example error return object: {
            "status": "error",
            "message": "An error occurred while uploading video.",
            "error": error_message,
            "driver": driver,
        }
        """
        result = youtube.create_video(
            self.driver,
            absolute_video_path,
            video_title,
            video_description,
            video_thumbnail_absolute_path,
            video_schedule_date,
            video_schedule_time,
        )
        self.driver = result['driver']
        return result

    def create_community_post(
        self, 
        community_post_title: str,
        schedule: Optional[dict] = None,
    ):
        """
        Function to create a community post on a Youtube channel.

        Parameters:
        - community_post_title (str): title of the community post.
        - schedule (Optional[dict]): time to schedule the post.
            The schedule object must follow the example below:
            {
                "date": "Apr 5, 2024",
                "time": "6:45 PM", # Only 15 minute increments (hour:0, hour:15, hour: 30, hour: 45)
                "GMT_timezone": "GMT+11" # Timezone of the schedule (GMT only)
            }

        Returns:
        - example success return object: {
            "status": "success", 
            "message": "Community post created successfully", 
            "driver": driver,
        }
        - example error return object: {
            "status": "error",
            "message": "An error occurred while creating community post."
            "error": error_message,
            "driver": driver
        }
        """
        result = youtube.create_community_post(
            self.driver,
            community_post_title,
            schedule,
        )
        self.driver = result['driver'] 
        return result

    def delete_channel(
        self,
        email: str,
    ):
        """
        Function to delete youtube channel permanently.

        Parameters:
        email (str): email of the channel.
        
        Returns:
        - example success return object: {
            "status": "success",
            "driver": driver,
            "message": "Channel deletion successful"
        }
        - example error return object: {
            "status": "error",
            "message": "An error occurred while deleting channel",
            "driver": driver,
            "error": str(e)
        }
        """
        result = youtube.delete_channel(self.driver, email)
        self.driver = result['driver']
        return result

    def delete_sub_channel(
        self,
        channel_name: str,
    ):
        """
        Function to delete sub youtube channel permanently.

        Parameters:
        channel_name (str): channel name of the channel.
        
        Returns:
        - example success return object: {
            "status": "success",
            "driver": driver,
            "message": "Channel deletion successful"
        }
        - example error return object: {
            "status": "error",
            "message": "An error occurred while deleting channel",
            "driver": driver,
            "error": str(e)
        }
        """
        result = youtube.delete_channel(self.driver, channel_name)
        self.driver = result['driver']
        return result

    def close(self):
        """
        Function to close Chromium driver. Call this function when you want the driver to be closed.

        Parameters:
        - None

        Returns (no errors):
        {
            "status": "success",
            "message": "Driver closed successfully"
        }
        """
        if self.driver:
            self.driver.quit()

            return {
                "status": "success",
                "message": "Driver closed successfully"
            }
        else:
            raise Exception("Driver not found")

    def create_sub_channels(
        self,
        sub_channels_names: List[str],
        ):
        """
        Function to create sub channels. This function will only work if the channel is already verified.

        Parameters:
        - sub_channels_names (List[str]): list of sub channels names to be created.
        """
        result = youtube.create_sub_channels(self.driver, sub_channels_names)

        if result:
            self.driver= result['driver']
            return result
        else:
            raise Exception("An error occurred while creating sub channels. Result is None.")

    def switch_to_sub_channel(
        self,
        channel_name: str, # You can also put the channel handle as the value for channel_name, will still work. That way, it will be more accurate if there is multiple channels with the same name.
    ):
        """
        Function to switch to a sub channel.
        Parameters:
        - channel_name (str): channel name or handle of the sub channel to switch to.
        Returns:
        - example success return object: {
            "status": "success",
            "message": "Switched to sub channel successfully.",
            "driver": driver
        }
        - example error return object: {
            "status": "error",
            "message": "An error occurred while switching to sub channel.",
            "error": str(e),
            "driver": driver
        }
        """
        result = youtube.switch_to_sub_channel(self.driver, channel_name)
        self.driver = result['driver']
        return result
    
    def list_all_channels(
        self
    ):
        """
        Function to list all channels for account.

        Parameters:
        - None

        Returns:
        - example success return object: {
            "channels_list": channels_list,
            "status": "success",
            "message": "Channels listed successfully",
        }
        - example error return object: {
            "status": "error",
            "message": str(e),
            "driver": driver
        }
        """
        result = youtube.list_all_channels(self.driver)
        self.driver = result['driver']
        return result

    def get_my_videos_stats(self):
        """
        Function to get all video stats of the channel.
        Parameters:
        - None
        Returns:
        - example success return object: {
            "status": "success",
            "message": "Video stats found.",
            "video_stats": video_stats,
            "driver": driver,
        }
        # Format of all_video_stats:
        [
            {
                "kind": "youtube#video",
                "etag": "eAGZdVWxoBjCKRPIjFKF64WuM7Q",
                "id": "0P1FGtdg8o8",
                "snippet": {
                    "publishedAt": "2024-04-03T17:00:02Z",
                    "channelId": "UCWsslCoN3b_wBaFVWK_ye_A",
                    "title": "What happened in Vegas",
                    "description": "Email list: https://hamza-ahmed.co.uk/UnfilteredWisdom\nThe only product I sell: https://www.skool.com/adonis\nInstagram: https://www.instagram.com/cultleaderhamza\nTwitter: https://twitter.com/HamzaAdonis",
                    "thumbnails": {
                        "default": {
                            "url": "https://i.ytimg.com/vi/0P1FGtdg8o8/default.jpg",
                            "width": 120,
                            "height": 90
                        },
                        "medium": {
                            "url": "https://i.ytimg.com/vi/0P1FGtdg8o8/mqdefault.jpg",
                            "width": 320,
                            "height": 180
                        },
                        "high": {
                            "url": "https://i.ytimg.com/vi/0P1FGtdg8o8/hqdefault.jpg",
                            "width": 480,
                            "height": 360
                        },
                        "standard": {
                            "url": "https://i.ytimg.com/vi/0P1FGtdg8o8/sddefault.jpg",
                            "width": 640,
                            "height": 480
                        },
                        "maxres": {
                            "url": "https://i.ytimg.com/vi/0P1FGtdg8o8/maxresdefault.jpg",
                            "width": 1280,
                            "height": 720
                        }
                    },
                    "channelTitle": "Hamza Ahmed",
                    "tags": [
                        "hamza ahmed",
                        "hamza",
                        "hamza97",
                        "self improvement",
                        "entrepreneur",
                        "entrepreneurship",
                        "beginners business",
                        "business",
                        "youtube subscribers",
                        "masculinity"
                    ],
                    "categoryId": "27",
                    "liveBroadcastContent": "none",
                    "defaultLanguage": "en-US",
                    "localized": {
                        "title": "What happened in Vegas",
                        "description": "Email list: https://hamza-ahmed.co.uk/UnfilteredWisdom\nThe only product I sell: https://www.skool.com/adonis\nInstagram: https://www.instagram.com/cultleaderhamza\nTwitter: https://twitter.com/HamzaAdonis"
                    },
                    "defaultAudioLanguage": "en-US"
                },
                "statistics": {
                    "viewCount": "31662",
                    "likeCount": "2255",
                    "favoriteCount": "0",
                    "commentCount": "582"
                }
            },
            ...
        ]
        - example error return object: {
            "status": "error",
            "message": "Video stats not found.",
            "driver": driver,
            "error": str(e),
        }
        """
        result = youtube.get_my_videos_stats(self.driver)
        self.driver = result['driver']
        return result
    
    def get_my_channel_handle(self):
        """
        Function to get channel handle of the channel.
        Parameters:
        - None
        Returns:
        - example success return object: {
            "status": "success",
            "message": "Channel ID found.",
            "channel_handle": channel_handle,
            "driver": driver
        }
        - example error return object: {
            "status": "error",
            "message": "Channel ID not found.",
            "driver": driver
        }
        """
        result = youtube.get_my_channel_handle(self.driver)
        self.driver = result['driver']
        return result

    def get_my_channel_id(self):
        """
        Function to get your channel id

        Parameters:
        - None

        Returns:
        - example success return object: {
            "status": "success",
            "channel_id": channel_id,
        }
        - example error return object: {
            "status": "error",
            "message": "An error occurred while converting handle to channel id.",
            "error": str(e),
        }
        """
        result = self.get_my_channel_handle()
        if result['status'] == 'success':
            channel_id_result = youtube.get_channel_id(result['channel_handle'])
            if channel_id_result['status'] == 'success':
                return channel_id_result
            else:
                return channel_id_result
        else:
            return result

    def get_my_channel_stats(self):
        """
        Function toe get your channel stats.

        Parameters:
        - None

        Returns:
        - example success return object: {
            "status": "success",
            "message": "Channel stats found.",
            "channel_stats": channel_stats,
            "driver": driver
        }
        - example error return object: {
            "status": "error",
            "message": "Channel stats not found.",
            "driver": driver
        }
        """
        channel_id_result = self.get_my_channel_id()
        if channel_id_result['status'] == 'success':
            result = youtube.get_channel_stats(channel_id_result['channel_id'])
            print(f"Channel stats: {result}")
            return result

@dataclass
class YoutubeData:
    def get_all_video_stats_from_channel(
        self,
        channel_handle: str,
    ):
        """
        Function to get all video stats and some extra information about the video, from a specific channel.

        Parameters:
        - channel_handle (str): channel handle of the channel to get all video stats from.

        Returns:
        - example success return object: {
            "all_video_stats": all_videos_with_stats,
            "status": "success",
            "message": "All video stats fetched successfully."
        }
        # Format of all_video_stats:
        [
            {
                "kind": "youtube#video",
                "etag": "eAGZdVWxoBjCKRPIjFKF64WuM7Q",
                "id": "0P1FGtdg8o8",
                "snippet": {
                    "publishedAt": "2024-04-03T17:00:02Z",
                    "channelId": "UCWsslCoN3b_wBaFVWK_ye_A",
                    "title": "What happened in Vegas",
                    "description": "Email list: https://hamza-ahmed.co.uk/UnfilteredWisdom\nThe only product I sell: https://www.skool.com/adonis\nInstagram: https://www.instagram.com/cultleaderhamza\nTwitter: https://twitter.com/HamzaAdonis",
                    "thumbnails": {
                        "default": {
                            "url": "https://i.ytimg.com/vi/0P1FGtdg8o8/default.jpg",
                            "width": 120,
                            "height": 90
                        },
                        "medium": {
                            "url": "https://i.ytimg.com/vi/0P1FGtdg8o8/mqdefault.jpg",
                            "width": 320,
                            "height": 180
                        },
                        "high": {
                            "url": "https://i.ytimg.com/vi/0P1FGtdg8o8/hqdefault.jpg",
                            "width": 480,
                            "height": 360
                        },
                        "standard": {
                            "url": "https://i.ytimg.com/vi/0P1FGtdg8o8/sddefault.jpg",
                            "width": 640,
                            "height": 480
                        },
                        "maxres": {
                            "url": "https://i.ytimg.com/vi/0P1FGtdg8o8/maxresdefault.jpg",
                            "width": 1280,
                            "height": 720
                        }
                    },
                    "channelTitle": "Hamza Ahmed",
                    "tags": [
                        "hamza ahmed",
                        "hamza",
                        "hamza97",
                        "self improvement",
                        "entrepreneur",
                        "entrepreneurship",
                        "beginners business",
                        "business",
                        "youtube subscribers",
                        "masculinity"
                    ],
                    "categoryId": "27",
                    "liveBroadcastContent": "none",
                    "defaultLanguage": "en-US",
                    "localized": {
                        "title": "What happened in Vegas",
                        "description": "Email list: https://hamza-ahmed.co.uk/UnfilteredWisdom\nThe only product I sell: https://www.skool.com/adonis\nInstagram: https://www.instagram.com/cultleaderhamza\nTwitter: https://twitter.com/HamzaAdonis"
                    },
                    "defaultAudioLanguage": "en-US"
                },
                "statistics": {
                    "viewCount": "31662",
                    "likeCount": "2255",
                    "favoriteCount": "0",
                    "commentCount": "582"
                }
            },
            ...
        ]
        - example error return object: {
            "status": "error",
            "message": "An error occurred. Please check errorr.txt for more details.",
            "error": str(e),
        }
        """
        result = youtube.get_all_video_stats_from_channel(channel_handle)
        return result

    def get_channel_id(
        self,
        channel_handle: str,
    ):
        """
        Function to get channel ID of the channel.

        Parameters:
        - channel_handle (str): channel handle of the channel to get channel ID from.

        Returns:
        - example success return object: {
            "status": "success",
            "channel_id": channel_id,
        }
        - example error return object: {
            "status": "error",
            "message": "An error occurred while converting handle to channel id.",
            "error": str(e),
        }
        """
        result = youtube.get_channel_id(channel_handle)
        return result

    def get_video_stats(
        self,
        video_id: str,
    ):
        """
        Function to get individual video stats

        Parameters:
        - video_id (str): video id for the video you want to get stats for.

        Returns:
        - example success return object: {
            "video_stats": video_stats,
            "status": "success"
        }
        - example error return object: {
            "status": "error",
            "message": "Error getting video stats.",
            "error": str(e),
        }
        """
        result = youtube.get_video_stats(video_id)
        return result
    
    def get_channel_stats(
        self,
        channel_id: str,
    ):
        """
        Function to get channel stats of the channel.

        Parameters:
        - channel_id (str): channel id of the channel to get stats from.

        Returns:
        - example success return object: {
            "status": "success",
            "channel_stats": channel_stats
        }
        - example error return object: {
            "status": "error",
            "messsage": "Error getting channel stats.",
            "error": str(e),
        }
        """

        result = youtube.get_channel_stats(channel_id)
        return result

    def download_video(
        self,
        video_id: str,
        absolute_path: str,
        filename: str
    ):
        """
        Function to download video from youtube.
        Parameters:
        - video_id (str): video id of the video to download.
        - absolute_path(str): absolute path to where you want to save the downloaded video.
        - filename(str): filename of the downloaded video.
        Returns:
        - example success return object: {
            "status": "success",
            "message": "Video downloaded successfully.",
        }
        - example error return object: {
            "status": "error",
            "message": "Error downloading video.",
            "error": str(e),
        }
        """
        result = youtube.download_video(video_id, absolute_path, filename)
        return result
    
    def download_thumbnail(
        self,
        video_id: str, 
        export_path: str, 
        thumbnail_name: str
    ):
        """
        Function to download thumbnail of a video from youtube, and save to specified absolute path.

        Parameters:
        - video_id (str): video id of the video to download thumbnail from.
        - export_path (str): absolute path to save the downloaded thumbnail.
        - thumbnail_name (str): name of the thumbnail to save as.

        Returns:
        - example success return object: {
            "status": "success",
            "message": "Thumbnail downloaded successfully.",
        }
        - example error return object: {
            "status": "error",
            "message": "Error downloading thumbnail.",
            "error": str(e),
        }
        """
        # # Download thumbnail and save to absolute path
        result = youtube.download_thumbnail(video_id, export_path, thumbnail_name)
        return result





# All methods for Youtube class:
from youtube_selenium_py.youtube.sign_into_youtube_channel import sign_into_youtube_channel
from youtube_selenium_py.youtube.create_channel import create_channel
from youtube_selenium_py.youtube.create_video import create_video 
from youtube_selenium_py.youtube.create_community_post import create_community_post
from youtube_selenium_py.youtube.delete_channel import delete_channel
from youtube_selenium_py.youtube.create_sub_channels import create_sub_channels
from youtube_selenium_py.youtube.switch_to_sub_channel import switch_to_sub_channel 
from youtube_selenium_py.youtube.edit_channel import edit_channel 
from youtube_selenium_py.youtube.list_all_channels import list_all_channels
from youtube_selenium_py.youtube.get_my_videos_stats import get_my_videos_stats
from youtube_selenium_py.youtube.get_my_channel_handle import get_my_channel_handle

# All methods for YoutubeData class:
from youtube_selenium_py.youtube.get_all_video_stats_from_channel import get_all_video_stats_from_channel
from youtube_selenium_py.youtube.get_channel_id import get_channel_id
from youtube_selenium_py.youtube.get_video_stats import get_video_stats
from youtube_selenium_py.youtube.get_channel_stats import get_channel_stats
from youtube_selenium_py.youtube.download_video import download_video 
from youtube_selenium_py.youtube.download_thumbnail import download_thumbnail




import unittest
from youtube_selenium_py.classes import YoutubeData 

class TestGetAllVideoStatsFromChannel(unittest.TestCase):

    def test_get_all_video_stats(self):

        youtubeData = YoutubeData()
        result = youtubeData.get_all_video_stats_from_channel("@Hamza97")

        if result:
            self.assertEqual(result["status"], "success")
            self.assertIsNotNone(result["all_video_stats"])
        else:
            raise Exception("Getting channel ID, result is None")



from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.by import By
from youtube_selenium_py.utils import find_element
from typing import Optional
import time

def create_community_post(
    driver,
    community_post_title: str,
    schedule: Optional[dict] = None,
):
    time.sleep(5)
    driver.get("https://youtube.com")
    try:
        print("1. Going to community page...")
        avatar_button = find_element(driver, By.CSS_SELECTOR, "button[id='avatar-btn']")
        avatar_button.click()

        view_channel_link = find_element(driver, By.XPATH, "//a[contains(text(), 'View your channel')]")
        view_channel_link.click()

        community_tab = find_element(driver, By.CSS_SELECTOR, "yt-tab-shape[tab-title='Community']")
        community_tab.click()

        channel_id = driver.current_url.split("youtube.com/")[1].split("/")[0]
        print(f"Channel ID: {channel_id}")
        
        print("2. Creating community post...")
        
        # Input community post title
        print(f"2.1. Inputting community post title: {community_post_title}...")
        community_post_title_input = find_element(driver, By.CSS_SELECTOR, "yt-formatted-string[id='commentbox-placeholder']")
        community_post_title_input.click()
        input_container = find_element(driver, By.CSS_SELECTOR, "div[id='contenteditable-root']")
        input_container.send_keys(community_post_title)

        if schedule:
            print(f"3. Scheduling post for {schedule['date']} at {schedule['time']} in {schedule['GMT_timezone']} timezone...")

            # row down button next to post button.
            print("3.1. Clicking on post additional options button...")
            post_additional_options_button = find_element(driver, By.CSS_SELECTOR, "button[aria-label='Action menu'][class='yt-spec-button-shape-next yt-spec-button-shape-next--filled yt-spec-button-shape-next--call-to-action yt-spec-button-shape-next--size-m yt-spec-button-shape-next--icon-button yt-spec-button-shape-next--segmented-end']")
            post_additional_options_button.click()

            print("3.2. Clicking on schedule post menu opener button...")
            schedule_post_menu_opener_button = find_element(driver, By.CSS_SELECTOR, "ytd-menu-service-item-renderer")
            schedule_post_menu_opener_button.click()
            
            print("3.3. Clicking on date picker button...")
            date_picker = find_element(driver, By.CSS_SELECTOR, "tp-yt-paper-button[id='date-picker']")
            date_picker.click()

            print("3.4. Inputting date...")
            date_input = find_element(driver, By.CSS_SELECTOR, "input[id='textbox']") 
            date_input.click()
            date_input.clear()
            date_input.send_keys(schedule["date"])
            date_input.send_keys(Keys.ENTER)

            print("3.5. Clicking on time picker button...")
            time_picker = find_element(driver, By.CSS_SELECTOR, "tp-yt-paper-button[id='time-picker']")
            time_picker.click()

            print("3.6. Selecting time...")
            time_listbox = find_element(driver, By.CSS_SELECTOR, "tp-yt-paper-listbox[id='time-listbox']")
            # Get all the tp-yt-paper-item in the time_listbox.
            all_times = time_listbox.find_elements(By.CSS_SELECTOR, "tp-yt-paper-item")
            
            for time_listitem in all_times:
                if time_listitem.text.strip() == schedule["time"].replace(" ", " "):
                    time_listitem.click()
                    break

            print("3.7. Selecting timezone...")
            timezone_picker = find_element(driver, By.CSS_SELECTOR, "tp-yt-paper-button[id='timezone-picker']")
            timezone_picker.click()

            timezone_listbox = find_element(driver, By.CSS_SELECTOR, "tp-yt-paper-listbox[id='timezone-listbox']")
            all_timezones_items = timezone_listbox.find_elements(By.CSS_SELECTOR, "tp-yt-paper-item")

            for timezone_list_item in all_timezones_items:
                timezone_GMT_format = timezone_list_item.text.strip().split("(")[1].split(")")[0]

                if timezone_GMT_format == schedule["GMT_timezone"]:
                    timezone_list_item.click()
                    break

            print("3.8. Clicking on schedule button...")
            schedule_button = find_element(driver, By.CSS_SELECTOR, "button[aria-label='Schedule']")
            schedule_button.click()

            try:
                print("3.9. Clicking on Got it button...")
                got_it_button = find_element(driver, By.CSS_SELECTOR, "button[aria-label='Got it']")
                got_it_button.click()

                schedule_button = find_element(driver, By.CSS_SELECTOR, "button[aria-label='Schedule']")
                schedule_button.click()
            except:
                print("Not needed to click on Got it button.")

        else:
            print("4. Posting community post...")
            post_button = find_element(driver, By.CSS_SELECTOR, "ytd-button-renderer[id='submit-button']")
            post_button.click()
    
        time.sleep(10)
        return {
            "status": "success", 
            "message": "Community post created successfully", 
            "driver": driver,
        }

    except Exception as e:
        print('Error:', e)
        with open("error.txt", "w") as f:
            f.write(str(e))

        return {
            "status": "error",
            "message": str(e),
            "driver": driver,
        }



import time
from selenium.webdriver.common.by import By
from youtube_selenium_py.utils import find_element, find_elements

def switch_to_sub_channel(
    driver,
    channel_name: str,
):
    try:
        time.sleep(5)
        if driver.current_url != "https://youtube.com":
            driver.get("https://youtube.com")

        print("1. Switching to sub channel...")
        avatar_button = find_element(driver, By.CSS_SELECTOR, "button[id='avatar-btn']")
        avatar_button.click()

        switch_account_button = find_element(driver, By.XPATH, "/html/body/ytd-app/ytd-popup-container/tp-yt-iron-dropdown/div/ytd-multi-page-menu-renderer/div[3]/div[1]/yt-multi-page-menu-section-renderer[1]/div[2]/ytd-compact-link-renderer[2]")
        switch_account_button.click()
        
        time.sleep(3)

        channels_items = find_elements(driver, By.CSS_SELECTOR, "ytd-account-item-renderer[thumbnail-size='48']")
        does_sub_channel_exist = False
        for channel_item in channels_items:
            if channel_name in channel_item.text:
                channel_item.click()
                does_sub_channel_exist = True
                break

        time.sleep(3)
        if not does_sub_channel_exist:
            return {
            "status": "error",
            "message": "Sub channel does not exist.",
            "driver": driver
            }
        return {
            "status": "success",
            "message": "Switched to sub channel successfully.",
            "driver": driver
        } 

    except Exception as e:
        return {
            "status": "error",
            "message": "An error occurred while switching to sub channel.",
            "error": str(e),
            "driver": driver
        }



import youtube_selenium_py.tests as tests
import unittest

tests = {
    "test_create_channel": tests.TestCreateChannel,
    "test_create_sub_channels": tests.TestCreateSubChannels,
    "test_edit_channel": tests.TestEditChannel,
    "test_edit_sub_channel": tests.TestEditSubChannel,
    "test_create_video": tests.TestCreateVideo,
    "test_create_video_sub_channel": tests.TestCreateVideoSubChannel,
    "test_create_community_post": tests.TestCreateCommunityPost,
    "test_create_community_post_sub_channel": tests.TestCreateCommunityPostSubChannel,
    "test_delete_channel": tests.TestDeleteChannel,
    "test_delete_sub_channel": tests.TestDeleteSubChannel,
    "test_switch_to_sub_channel": tests.TestSwitchToSubChannel,
    "test_list_all_channels": tests.TestListAllChannels,
    "test_get_my_videos_stats": tests.TestGetMyVideosStats,
    "test_get_all_video_stats_from_channel": tests.TestGetAllVideoStatsFromChannel,
    "test_get_video_stats": tests.TestGetVideoStats,
    "test_get_my_channel_handle": tests.TestGetMyChannelHandle,
    "test_get_my_channel_id": tests.TestGetMyChannelID,
    "test_get_channel_id": tests.TestGetChannelID,
    "test_get_my_channel_stats": tests.TestGetMyChannelStats,
    "test_get_channel_stats": tests.TestGetChannelStats,
    "test_download_video": tests.TestDownloadVideo,
    "test_download_thumbnail": tests.TestDownloadThumbnail,
}

def main():
    # Option 0: Run all tests
    print("0. Run all tests")
    for index, test in enumerate(tests):
        print(f"{index+1}. {test}")
    choice = int(input("Enter the number of the test you want to run: "))
    if choice == "0":
        # run all tests
        suite = unittest.TestSuite()
        # continue
        suite.addTests([unittest.TestLoader().loadTestsFromTestCase(test) for test in tests.values()])
        unittest.TextTestRunner().run(suite)
    test = list(tests.values())[choice-1]
    test = unittest.TestLoader().loadTestsFromTestCase(test)
    unittest.TextTestRunner().run(test)

main()



from selenium.webdriver.common.by import By
from youtube_selenium_py.utils import find_element
import time

def delete_channel(
    driver,
    email: str,
):
    try:
        time.sleep(5)
        print("1. Going to advanced account settings page...")
        driver.get("https://www.youtube.com/account_advanced")

        print("2. Going to delete channel page...")
        # Get the anchor element with the text "Delete channel"
        delete_channel_link = find_element(driver, By.XPATH, "//a[text()='Delete channel']")
        delete_channel_link.click()

        print("3. Clicking on 'permanently delete' dropdown button...")
        try:
            permanently_delete_button = find_element(driver, By.CSS_SELECTOR, "button[id='sectioni8']")
            permanently_delete_button.click()
        except:
            permanently_delete_button = find_element(driver, By.CSS_SELECTOR, "button[id='sectioni6']")
            permanently_delete_button.click()
        
        print("4. Checking off both checkboxes...")
        checkbox_1 = find_element(driver, By.XPATH, "/html/body/c-wiz/div/div[2]/div[2]/c-wiz/div/div[5]/div[2]/div/div[1]/div/div[1]/div/input")
        checkbox_1.click()

        checkbox_2 = find_element(driver, By.XPATH, "/html/body/c-wiz/div/div[2]/div[2]/c-wiz/div/div[5]/div[2]/div/div[2]/div/div[1]/div/input")
        checkbox_2.click()

        print("5. Clicking on delete button...")
        delete_button = find_element(driver, By.XPATH, "/html/body/c-wiz/div/div[2]/div[2]/c-wiz/div/div[5]/div[2]/div/div[3]/div/div/button")
        delete_button.click()

        if not "@" in email:
            print("6. Entering channel name...")

        else:
            print("6. Entering email address...")

        email_address_input = find_element(driver, By.XPATH, "/html/body/div[13]/div[2]/div/div[1]/div/div[1]/div/div/label/input")
        email_address_input.send_keys(email)
        
        print("7. Clicking on final delte confirmation button...")
        delete_content_button = find_element(driver, By.XPATH, "/html/body/div[13]/div[2]/div/div[2]/div[2]/button")
        delete_content_button.click()

        time.sleep(10)

        return {
            "status": "success",
            "driver": driver,
            "message": "Channel deletion successful"
        }

    except Exception as e:

        with open("error.txt", "w") as f:
            f.write(str(e))

        print("Error occurred with deleting channel.")

        return {
            "status": "error",
            "message": "An error occurred while deleting channel",
            "driver": driver,
            "error": str(e)
       }



# Youtube Class
from youtube_selenium_py.tests.test_create_channel import TestCreateChannel
from youtube_selenium_py.tests.test_create_sub_channels import TestCreateSubChannels

from youtube_selenium_py.tests.test_create_video import TestCreateVideo 
from youtube_selenium_py.tests.test_create_video_sub_channel import TestCreateVideoSubChannel

from youtube_selenium_py.tests.test_create_community_post import TestCreateCommunityPost
from youtube_selenium_py.tests.test_create_community_post_sub_channel import TestCreateCommunityPostSubChannel

from youtube_selenium_py.tests.test_delete_channel import TestDeleteChannel
from youtube_selenium_py.tests.test_delete_sub_channel import TestDeleteSubChannel

from youtube_selenium_py.tests.test_edit_channel import TestEditChannel
from youtube_selenium_py.tests.test_edit_sub_channel import TestEditSubChannel

from youtube_selenium_py.tests.test_switch_to_sub_channel import TestSwitchToSubChannel
from youtube_selenium_py.tests.test_list_all_channels import TestListAllChannels 

from youtube_selenium_py.tests.test_get_my_videos_stats import TestGetMyVideosStats

from youtube_selenium_py.tests.test_get_my_channel_handle import TestGetMyChannelHandle
from youtube_selenium_py.tests.test_get_my_channel_id import TestGetMyChannelID

from youtube_selenium_py.tests.test_get_my_channel_stats import TestGetMyChannelStats

# YoutubeData Class
from youtube_selenium_py.tests.test_get_all_video_stats_from_channel import TestGetAllVideoStatsFromChannel
from youtube_selenium_py.tests.test_get_channel_id import TestGetChannelID 
from youtube_selenium_py.tests.test_get_video_stats import TestGetVideoStats
from youtube_selenium_py.tests.test_get_channel_stats import TestGetChannelStats
from youtube_selenium_py.tests.test_download_video import TestDownloadVideo
from youtube_selenium_py.tests.test_download_thumbnail import TestDownloadThumbnail



from youtube_selenium_py.youtube.list_all_channels import list_all_channels

def get_my_channel_handle(driver):
    try:
        result = list_all_channels(driver)
        if result["status"] == "success":
            channel_handle = result["channels_list"][0]['channel_handle']
        else:
            raise Exception("Error: channel listing not successful.")
        
        print(f"My Channel Handle: {channel_handle}")
        return {
            "status": "success",
            "message": "Channel ID found.",
            "channel_handle": channel_handle,
            "driver": driver
        }
    except Exception as e:
        print("Error: " + str(e))

        with open("error.txt", "a") as error_file:
            error_file.write(str(e) + "\n")

        return {
            "status": "error",
            "message": "Channel ID not found.",
            "driver": driver
        }



from typing import Optional
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.by import By
from youtube_selenium_py.utils import find_element
import time

def create_video(
    driver,
    absolute_video_path: str,
    video_title: str,
    video_description: str,
    video_thumbnail_absolute_path: Optional[str] = None,
    video_schedule_date: Optional[str] = None,
    video_schedule_time: Optional[str] = None,
):
    try: 
        print("1. Navigate to the upload page...")
        time.sleep(3)
        driver.get("https://youtube.com/upload")

        print("2. Uploading the video...")
        upload_video_input = find_element(driver, By.CSS_SELECTOR, "input[type='file']")
        upload_video_input.send_keys(absolute_video_path)
        
        print(f"3.1. Adding video title '{video_title}'...")
        # Input title and description of video
        title_input = find_element(driver, By.CSS_SELECTOR, "div[aria-label='Add a title that describes your video (type @ to mention a channel)']")
        # set_element_innertext(driver, title_input, video_title)
        title_input.click()
        title_input.send_keys(Keys.CONTROL + "a")
        title_input.send_keys(Keys.DELETE)
        title_input.send_keys(video_title)
        
        print(f"3.2. Adding video description '{video_description}'...")
        description_input = find_element(driver, By.CSS_SELECTOR, "div[aria-label='Tell viewers about your video (type @ to mention a channel)']")
        description_input.click()
        description_input.send_keys(Keys.CONTROL + "a")
        description_input.send_keys(Keys.DELETE)
        description_input.send_keys(video_description)

        # Upload thumbnail if video thumbnail is specified
        if video_thumbnail_absolute_path:
            print("Uploading video thumbnail...")
            thumbnail_input = find_element(driver, By.CSS_SELECTOR, "input[type='file']")
            thumbnail_input.send_keys(video_thumbnail_absolute_path)
            time.sleep(10)
        
        print("3.3. Setting video visibility to 'Not Made for Kids'...")
        not_for_kids_radio = find_element(driver, By.CSS_SELECTOR , "tp-yt-paper-radio-button[name='VIDEO_MADE_FOR_KIDS_NOT_MFK']", 100)
        not_for_kids_radio.click()

        print(f"3.4. Setting video visibility to 'Public'...")
        visibility_tab = find_element(driver, By.CSS_SELECTOR, "button[id='step-badge-3']")
        visibility_tab.click()

        video_public_radio = find_element(driver, By.CSS_SELECTOR, "tp-yt-paper-radio-button[name='PUBLIC']")
        video_public_radio.click()

        # Schedule video if schedule time is specified
        if video_schedule_date and video_schedule_time:
            print(f"3.5. Scheduling video for {video_schedule_date} at {video_schedule_time}...")
            schedule_dropdown_button = find_element(driver, By.CSS_SELECTOR, "ytcp-icon-button[id='second-container-expand-button']")
            schedule_dropdown_button.click()
       
            date_toggle_menu = find_element(driver, By.CSS_SELECTOR, "ytcp-text-dropdown-trigger[id='datepicker-trigger']")
            date_toggle_menu.click()

            date_input = find_element(driver, By.XPATH, "/html/body/ytcp-date-picker/tp-yt-paper-dialog/div/form/tp-yt-paper-input/tp-yt-paper-input-container/div[2]/div/iron-input/input")
            date_input.clear()
            date_input.click()
            date_input.send_keys(video_schedule_date)
            date_input.send_keys(Keys.ENTER)

            time_toggle_menu = find_element(driver, By.XPATH, "/html/body/ytcp-uploads-dialog/tp-yt-paper-dialog/div/ytcp-animatable[1]/ytcp-uploads-review/div[2]/div[1]/ytcp-video-visibility-select/div[3]/div[2]/ytcp-visibility-scheduler/div[1]/ytcp-datetime-picker/div/div[2]/form/ytcp-form-input-container/div[1]/div/tp-yt-paper-input/tp-yt-paper-input-container/div[2]/div/iron-input/input")
            time_toggle_menu.click()

            time_menu = find_element(driver, By.XPATH, "/html/body/ytcp-time-of-day-picker/tp-yt-paper-dialog/tp-yt-paper-listbox")
            # Look through all the children of the time menu (tp-yt-paper-item), and select the one with the same text as the time input.
            for child in time_menu.find_elements(By.CSS_SELECTOR, "tp-yt-paper-item"):
                if child.text == video_schedule_time.replace(" ", " "):
                    child.click()
                    break

        print("4. Checking if the video was uploaded...")
        while True:
            time.sleep(1)
            progress_label = find_element(driver, By.CSS_SELECTOR, "span[class='progress-label style-scope ytcp-video-upload-progress']")
            if not "Uploading" in progress_label.text:
                break
            print("Still uploading...")

        print("5. Saving the video...")
        save_button = find_element(driver, By.CSS_SELECTOR, "ytcp-button[id='done-button']")
        save_button.click()

        try:
            # Close the close button if it exists
            close_button = find_element(driver, By.CSS_SELECTOR, "ytcp-button[id='close-button']")
            close_button.click()
        except:
            print("Close button didn't exist, no need to close it.")

        time.sleep(5)

        print("6. Getting the video and channel id...")
        # Get channel id
        current_url = driver.current_url
        channel_id = current_url.split("channel/")[1].split("/")[0]

        # if Welcome popup is active, click continue button
        try:
            continue_button = find_element(driver, By.CSS_SELECTOR, "ytcp-button[id='dismiss-button']")
            continue_button.click()
        except:
            print("No need to close welcome popup, it doesn't show up at all.")
        
        # Get video id
        video_href = find_element(driver, By.CSS_SELECTOR, "a[id='video-title']").get_attribute("href")

        video_id = ""

        if video_href:
            video_id = video_href.split("video/")[1].split("/")[0]

        print("7. Video uploaded successfully!")

        return_dict = {
            "status": "success",
            "channel_id": channel_id,
            "video_id": video_id,
            "message": "Video uploaded successfully",
            "driver": driver,
        }

        return return_dict

    except Exception as e:
        print('Error:', e)
        with open("error.txt", "w") as f:
            f.write(str(e))

        return {
            "status": "error",
            "message": "An error occurred while uploading video.",
            "error": str(e),
            "driver": driver,
        }



import unittest
from youtube_selenium_py.classes import YoutubeData 

class TestGetChannelID(unittest.TestCase):

    def test_get_channel_id_success(self):

        youtubeData = YoutubeData()
        result = youtubeData.get_channel_id("@Hamza97")
        if result:
            self.assertEqual(result["status"], "success")
            self.assertIsNotNone(result["channel_id"])
        else:
            raise Exception("Getting channel ID, result is None")



import unittest
from youtube_selenium_py.classes import Youtube
import os
from dotenv import load_dotenv

load_dotenv()

test_email = os.getenv("TEST_EMAIL")
test_password = os.getenv("TEST_PASSWORD")

class TestCreateSubChannels(unittest.TestCase):

    def test_create_video_success(self):
        # Calling the create_video function with valid parameters
        youtube = Youtube(email=test_email, password=test_password)

        result = youtube.create_sub_channels(
            sub_channels_names=["John C", "Jack S", "Lupepe T"]
        )

        # Asserting that the result is a success
        self.assertEqual(result["status"], "success")
        youtube.close()




import unittest
from youtube_selenium_py.classes import Youtube
import os
from dotenv import load_dotenv

load_dotenv()
test_email = os.getenv("TEST_EMAIL") or ""
test_password = os.getenv("TEST_PASSWORD") or ""


class TestSwitchToSubChannel(unittest.TestCase):

    def test_switch_to_sub_channel_success(self):
        # Calling the create_video function with valid parameters

        youtube = Youtube(email=test_email, password=test_password)
        result = youtube.switch_to_sub_channel(
            "Adonis Here"
        )

        if result:
            # Asserting that the result is a success
            self.assertEqual(result["status"], "success")
        else:
            raise Exception("Channel switching failed, result is None")

        youtube.close()



from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.by import By
from youtube_selenium_py.utils import find_element, scroll_to_bottom, set_element_innertext, gen_random_string
import time

def edit_channel(
    driver,
    channel_name: str,
    channel_handle: str,
    channel_description: str,
    profile_picture_path: str,
    banner_picture_path: str, watermark_picture_path: str,
    contact_email_path: str,
    links: list,
):
    try:
        print("1. Navigating to channel page...")
        avatar_button = find_element(driver, By.CSS_SELECTOR, "button[id='avatar-btn']")
        avatar_button.click()
        
        # Get anchor element with text "View your channel"
        view_channel_link = find_element(driver, By.XPATH, "//a[contains(text(), 'View your channel')]")
        view_channel_link.click()

        print("2. Getting channel ID...")
        while True:
            current_url = driver.current_url
            if current_url.find("channel/") != -1:
                channel_id = current_url.split("/")[-1]
                if "?" in channel_id:
                    channel_id = channel_id.split("?")[0]
                break

        print("3. Navigating to basic info tab...")
        driver.get(f"https://studio.youtube.com/channel/{channel_id}/editing/details")

        # If Welcome to Youtube Studio popup opens:
        try:
            continue_button = find_element(driver, By.XPATH, '//ytcp-button[.//div[text()="Continue"]]')
            continue_button.click()
        except:
            print("No welcome popup found.")

        print("4. Filling in channel details...")
        channel_name_input = find_element(driver, By.CSS_SELECTOR, "input[id='brand-name-input']")
        channel_name_input.click()
        channel_name_input.send_keys(Keys.CONTROL + 'a')
        channel_name_input.send_keys(Keys.CONTROL + Keys.BACKSPACE)
        channel_name_input.send_keys(channel_name)

        channel_handle_input = find_element(driver, By.CSS_SELECTOR, "input[id='handle-input']")
        channel_handle_input.click()
        channel_handle_input.send_keys(Keys.CONTROL + 'a')
        channel_handle_input.send_keys(Keys.CONTROL + Keys.BACKSPACE)
        channel_handle_input.send_keys(channel_handle)

        time.sleep(5)
        while True:
            try:
                # Check if the text "This handle isn't available" is somewhere visible on the screen
                if find_element(driver, By.XPATH, '//*[contains(text(), "This handle isn\'t available.")]'):
                    channel_handle  = gen_random_string(9)
                    channel_handle_input = find_element(driver, By.CSS_SELECTOR, "input[id='handle-input']")
                    channel_handle_input.click()
                    channel_handle_input.clear()
                    channel_handle_input.send_keys(channel_handle)
                    time.sleep(3)
                else:
                    break
            except:
                break

        channel_description_input = find_element(driver, By.CSS_SELECTOR, "div[aria-label='Tell viewers about your channel. Your description will appear in the About section of your channel and search results, among other places.']")
        set_element_innertext(driver, channel_description_input, channel_description)

        print("5. Filling in contact channel links...")
        scroll_to_bottom(driver)

        for i, link in enumerate(links):
            print(f"[{i+1}] Adding link...")
            add_link_button = find_element(driver, By.ID, 'add-link-button')
            add_link_button.click()

            print(f"[{i+1}] Adding title: {link['title']}")
            # Find all title inputs
            title_inputs = driver.find_elements(By.CSS_SELECTOR, 'input[placeholder="Enter a title"]')

            # Iterate through all title inputs, to get the emtpy input to send the title 
            for title_input in title_inputs:
                # Check if the input has no value or has not been typed in
                if not title_input.get_attribute('value'):
                    title_input.click()
                    title_input.send_keys(link['title'])

            print(f"[{i+1}] Adding URL: {link['url']}")
            # Find all url inputs
            url_inputs = driver.find_elements(By.CSS_SELECTOR, 'input[placeholder="Enter a URL"]')

            for url_input in url_inputs:
                # Check if the input has no value or has not been typed in
                if not url_input.get_attribute('value'):
                    url_input.click()
                    url_input.send_keys(link['url'])

        scroll_to_bottom(driver)

        print("6. Filling in contact email...")
        contact_email_input = find_element(driver, By.CSS_SELECTOR, 'input[placeholder="Email address"]')
        contact_email_input.click()
        contact_email_input.send_keys(contact_email_path)

        print("7. Navigating to branding tab...")
        branding_navigation_button = find_element(driver, By.XPATH, "//ytcp-ve[span[contains(text(), 'Branding')]]")
        branding_navigation_button.click()
        
        print("8. Uploading profile picture, banner, and watermark...")
        # Upload profile picture
        thumbnail_upload_file_input = find_element(driver, By.XPATH, "/html/body/ytcp-app/ytcp-entity-page/div/div/main/div/ytcp-animatable[6]/ytcp-channel-editing-section/iron-pages/div[2]/ytcp-channel-editing-images-tab/div/section[1]/ytcp-profile-image-upload/div/div[3]/div[2]/div[2]/input")
        thumbnail_upload_file_input.send_keys(profile_picture_path)
        done_button = find_element(driver, By.XPATH, "/html/body/ytcp-profile-image-editor/ytcp-dialog/tp-yt-paper-dialog/div[3]/div/ytcp-button[2]")
        done_button.click()

        # Upload banner
        banner_upload_file_input = find_element(driver, By.XPATH, "/html/body/ytcp-app/ytcp-entity-page/div/div/main/div/ytcp-animatable[6]/ytcp-channel-editing-section/iron-pages/div[2]/ytcp-channel-editing-images-tab/div/section[2]/ytcp-banner-upload/div/div[3]/div[2]/div[2]/input")
        banner_upload_file_input.send_keys(banner_picture_path)
        done_button = find_element(driver, By.XPATH, "/html/body/ytcp-banner-editor/ytcp-dialog/tp-yt-paper-dialog/div[3]/div/ytcp-button[2]")
        done_button.click()

        # scroll to bottom
        scroll_to_bottom(driver)

        # Upload watermark
        watermark_upload_file_input = find_element(driver, By.XPATH, "/html/body/ytcp-app/ytcp-entity-page/div/div/main/div/ytcp-animatable[6]/ytcp-channel-editing-section/iron-pages/div[2]/ytcp-channel-editing-images-tab/div/section[3]/ytcp-video-watermark-upload/div/div[3]/div[2]/div[2]/input")
        watermark_upload_file_input.send_keys(watermark_picture_path)
        done_button = find_element(driver, By.XPATH, "/html/body/ytcp-video-watermark-image-editor/ytcp-dialog/tp-yt-paper-dialog/div[3]/div/ytcp-button[2]")
        done_button.click()

        print("9. Publishing channel...")
        time.sleep(3)
        publish_button = find_element(driver, By.XPATH, '//ytcp-button[@id="publish-button"]')
        publish_button.click()
        time.sleep(30)

        print("10. Channel edited successfully.")
        return {
            "status": "success",
            "message": "Channel edited successfully",
            "channel_id": channel_id,
            "channel_name": channel_name,
            "channel_handle": channel_handle,
            "driver": driver,
        }

    except Exception as e:
        print("Error: ", e)
        with open("error.txt", "w") as file:
            file.write(str(e))
        return {
            "status": "error",
            "message": "An error occurred while editing channel.",
            "error": str(e),
            "driver": driver,
        }



import unittest
from youtube_selenium_py.classes import Youtube
from dotenv import load_dotenv
import os

load_dotenv()

test_email = os.getenv("TEST_EMAIL")
test_password = os.getenv("TEST_PASSWORD")

class TestGetMyVideosStats(unittest.TestCase):

    def get_my_videos_stats_success(self):

        youtube = Youtube(email=test_email, password=test_password)
        result = youtube.get_my_videos_stats()
        if result:
            self.assertEqual(result["status"], "success")
            self.assertIsNotNone(result["video_stats"])
        else:
            raise Exception("Getting my video stats failed, result is None")

        youtube.close()



import requests
        
def get_channel_stats(channel_id: str):
    print("Getting channel stats...")
    try:
        url = f"https://yt.lemnoslife.com/noKey/channels?id={channel_id}&part=snippet,statistics"
        response = requests.get(url)
        channel_stats = response.json()['items'][0]
            
        print("Successfully got channel stats.")
        return {
            "status": "success",
            "channel_stats": channel_stats
        }

    except Exception as e:
        print("Error getting channel stats...")

        with open("error.txt", "w") as f:
            f.write(f"Error getting channel stats: {str(e)}\n")

        return {
            "status": "error",
            "messsage": "Error getting channel stats.",
            "error": str(e),
        }




import requests
from youtube_selenium_py.youtube import get_video_stats

def download_thumbnail(video_id, export_path, thumbnail_name):
    try:
        result = get_video_stats(video_id)
        if result["status"] == "success":
            video_data = result["video_stats"]
        else:
            print("Error: Unable to fetch video data.")
            return result 

        snippet = video_data["snippet"]
        thumbnail_url = snippet["thumbnails"]["maxres"]["url"]

        response = requests.get(thumbnail_url)
        with open(f"{export_path}/{thumbnail_name}.jpg", 'wb') as file:
            file.write(response.content)
        print(f"Thumbnail '{thumbnail_name}.jpg' downloaded at {export_path}")

        return {
            "status": "success",
        }
    except Exception as e:
        print(f"Error: {e}")
        return {
            "status": "error",
            "message": str(e)
        }



import unittest
import os
from youtube_selenium_py.classes import Youtube
import time
from dotenv import load_dotenv

load_dotenv()

test_email = os.getenv("TEST_EMAIL")
test_password = os.getenv("TEST_PASSWORD")

# Convert to absolute path
profile_picture_path = os.path.abspath("./tests/assets/channel_logo.jpg")
banner_picture_path = os.path.abspath("./tests/assets/channel_banner.png")
watermark_picture_path = os.path.abspath("./tests/assets/channel_logo.jpg")

class TestEditChannel(unittest.TestCase):

    def test_edit_channel_success(self):
        if not test_email or not test_password:
            self.skipTest("Test email and password not provided")

        youtube = Youtube(email=test_email, password=test_password)
        result = youtube.edit_channel(
            channel_name="Adonis Jamal",
            channel_handle="jamal283492857",
            channel_description="hello there",
            profile_picture_path=profile_picture_path,
            banner_picture_path=banner_picture_path,
            watermark_picture_path=watermark_picture_path,
            contact_email_path="contact@williamferns.com",
            links=[{"title": "Link 1", "url": "https://www.link1.com"}, {"title": "Link 2", "url": "https://www.link2.com"}]
        )

        if result:
            print(result)
            self.assertEqual(result["message"], "Channel edited successfully")
            self.assertIsNotNone(result["channel_id"])
            self.assertIsNotNone(result["channel_name"])
            self.assertIsNotNone(result["channel_handle"])
        else:
            raise Exception("Channel edit failed, result is None")
        
        time.sleep(5)
        youtube.close()



from typing import Optional
import undetected_chromedriver as uc
from youtube_selenium_py.utils import find_element
import time
from selenium.webdriver.common.by import By

def sign_into_youtube_channel(
    driver,
    email: Optional[str] = None,
    password: Optional[str] = None, 
    cookies: Optional[list] = None,
    absolute_chromium_profile_path: Optional[str] = None
):
    if not email and not password and not cookies and not absolute_chromium_profile_path:
        raise Exception("You must provide either an email and password, chromium driver path, or cookies.")

    driver.get("https://youtube.com")
    
    try:
        if cookies:
            for cookie in cookies:
                driver.add_cookie(cookie)
            driver.refresh()
            return driver
        
        elif absolute_chromium_profile_path:
            driver = uc.Chrome(absolute_chromium_profile_path)
            driver.get("https://youtube.com")
            return driver

        elif email and password:
            # Sign up button
            sign_in_with_google_link = find_element(driver, By.CSS_SELECTOR, "a[href*='https://accounts.google.com/ServiceLogin']")
            link = sign_in_with_google_link.get_attribute("href")

            if link is None: 
                raise Exception("Could not find sign in with google link.")

            driver.get(link)
       
            email_input = find_element(driver, By.CSS_SELECTOR, "input[type='email']")
            email_input.click()
            email_input.send_keys(email)

            next_button = find_element(driver, By.XPATH, "//button[contains(span/text(), 'Next')]")
            next_button.click()

            time.sleep(3)

            password_input = find_element(driver, By.CSS_SELECTOR, "input[type='password']")
            password_input.click()
            password_input.send_keys(password)

            see_password_checkbox = find_element(driver, By.CSS_SELECTOR, "input[type='checkbox']")
            see_password_checkbox.click()

            next_button = find_element(driver, By.XPATH, "//button[contains(span/text(), 'Next')]")
            next_button.click()

            print("Implicitly sleeping for 20 seconds, if your account has two setp authentication, or asking for verification code...")
            time.sleep(20)
            return driver
        else:
            raise Exception("You must provide either an email and password, chromium driver path, or cookies.")

    except Exception as e:
        print("Error signing into youtube channel: ", e)

        with open("error.txt", "w") as f:
            f.write(str(e))

        return driver



import unittest
from youtube_selenium_py.classes import Youtube
from dotenv import load_dotenv
import os

load_dotenv()

test_email = os.getenv("TEST_EMAIL")
test_password = os.getenv("TEST_PASSWORD")

class TestGetMyChannelHandle(unittest.TestCase):

    def get_my_channel_handle_success(self):

        youtube = Youtube(email=test_email, password=test_password)
        result = youtube.get_my_channel_handle()
        if result:
            self.assertEqual(result["status"], "success")
        else:
            raise Exception("Getting channel handle, result is None")

        youtube.close()



from typing import Any, Dict
import requests

def get_video_stats(video_id: str):
    try:
        url=f"https://yt.lemnoslife.com/noKey/videos?part=snippet,statistics&id={video_id}"

        response = requests.get(url)
        response_json = response.json()
        video_stats : Dict[Any, Any] = response_json['items'][0]
        print("Successfully fetched video statistics.")

        return {
            "video_stats": video_stats,
            "status": "success"
        }

    except Exception as e:
        print(f"Error: {e}")

        with open("error.txt", "w") as f:
            f.write(str(e))

        return {
            "status": "error",
            "message": "Error getting video stats.",
            "error": str(e),
        }



from setuptools import setup, find_packages

# Package metadata
NAME = 'youtube_selenium_py'
VERSION = '1.0.6'
DESCRIPTION = "A Python package to create youtube channels, sub channels, upload videos, create community posts, edit channel, delete channel, and so much more."
URL = 'https://github.com/Automa-Automations/youtube_selenium_py'
AUTHOR = 'William Ferns'
AUTHOR_EMAIL = 'business@agnostica.site'
LICENSE = 'MIT'
CLASSIFIERS = [
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
]
KEYWORDS = ['youtube', 'API', 'video', 'channel', 'upload', 'statistics', 'python', 'development', 'selenium']
PROJECT_URLS = {
    'Bug Reports': 'https://github.com/Automa-Automations/youtube_selenium_py/issues',
    'Source': 'https://github.com/Automa-Automations/youtube_selenium_py',
    'Automa Automations': 'https://github.com/Automa-Automations',
}

# Package requirements
with open('requirements.txt') as f:
    INSTALL_REQUIRES = f.read().splitlines()

# Package setup
setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url=URL,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    license=LICENSE,
    classifiers=CLASSIFIERS,
    keywords=KEYWORDS,
    project_urls=PROJECT_URLS,
    packages=find_packages(),
    include_package_data=True,
    install_requires=INSTALL_REQUIRES,
)



import unittest
import os
from youtube_selenium_py.classes import Youtube
import time
from dotenv import load_dotenv

load_dotenv()

test_email = os.getenv("TEST_EMAIL")
test_password = os.getenv("TEST_PASSWORD")

class TestCreateChannel(unittest.TestCase):

    def test_create_channel_success(self):
        if not test_email or not test_password:
            self.skipTest("Test email and password not provided")

        youtube = Youtube(email=test_email, password=test_password)
        # Test the create_channel function with valid inputs
        result = youtube.create_channel()

        if result:
            print(result)
            # Assert that the channel was created successfully
            self.assertEqual(result["message"], "Channel created successfully")
            self.assertIsNotNone(result["channel_id"])
            self.assertIsNotNone(result["cookies"])
            self.assertIsNotNone(result["cookies"])
            self.assertIsNotNone(result["channel_name"])
            self.assertIsNotNone(result["channel_handle"])
        else:
            raise Exception("Channel creation failed, result is None")
        
        time.sleep(5)
        youtube.close()



from selenium.webdriver.common.by import By
from youtube_selenium_py.utils import find_element
import time

def create_channel(
    driver,
):

    try:
        print("1. Creating channel...")
        youtube_url = "https://youtube.com"
        time.sleep(5)
        if driver.current_url != youtube_url:
            driver.get("https://youtube.com")

        avatar_button = find_element(driver, By.CSS_SELECTOR, "button[id='avatar-btn']")
        avatar_button.click()

        create_channel_link = find_element(driver, By.XPATH, "//a[contains(text(), 'Create a channel')]")
        create_channel_link.click()
        time.sleep(5)
        all_inputs = driver.find_elements(By.CSS_SELECTOR, "input[class='style-scope tp-yt-paper-input']")
        default_channel_name = all_inputs[0].get_attribute("value")
        default_channel_handle = all_inputs[1].get_attribute("value")

        create_channel_button = find_element(driver, By.XPATH, "//button[contains(., 'Create channel')]")
        create_channel_button.click()

        print("2. Getting channel ID...")
        while True:
            current_url = driver.current_url
            if current_url.find("channel/") != -1:
                channel_id = current_url.split("/")[-1]
                if "?" in channel_id:
                    channel_id = channel_id.split("?")[0]
                break

        # Data to be returned
        data = {
            "status": "success",
            "channel_id": channel_id, 
            "channel_name": default_channel_name,
            "channel_handle": default_channel_handle,
            "message": "Channel created successfully", 
            "cookies": driver.get_cookies(),
            "driver": driver
        }

        print("10. Channel created successfully.")
        print("Return Data: ", data)
        return data

    except Exception as e:
        print("Error: ", e)
        with open("error.txt", "w") as file:
            file.write(str(e))
        return {
            "status": "error",
            "message": "An error occurred while creating channel.",
            "error": str(e),
            "driver": driver,
        }



import unittest
from youtube_selenium_py.classes import YoutubeData

class TestGetChannelStats(unittest.TestCase):
    def test_get_channel_stats_success(self):
        youtube_data = YoutubeData()
        print("in get_channel_stats_success")
        channel_id_result = youtube_data.get_channel_id("@Hamza97")
        if channel_id_result['status'] == 'success':
            result = youtube_data.get_channel_stats(channel_id_result['channel_id'])
            print(result)
            self.assertEqual(result["status"], "success")
        else:
            raise Exception("Getting channel id, result is None")
